from pathlib import Path
from os import path
import peewee as pw

db_path = Path().absolute() / "data" / "who@me" / "data.db"
db_path.parent.mkdir(exist_ok=True, parents=True)
db = pw.SqliteDatabase(db_path)


class MainTable(pw.Model):
    operator_id = pw.IntegerField()
    operator_name = pw.CharField()
    target_id = pw.IntegerField()
    group_id = pw.IntegerField()
    time = pw.CharField()
    message = pw.TextField()
    message_id = pw.IntegerField()

    class Meta:
        database = db
        primary_key = pw.CompositeKey(
            "operator_id",
            "operator_name",
            "target_id",
            "group_id",
            "time",
            "message",
            "message_id",
        )


if not path.exists(db_path):
    db.connect()
    db.create_tables([MainTable])
    db.close()



import time
from typing import List
import nonebot
from nonebot import on_command, on_message, on_regex
from nonebot.adapters.onebot.v11 import (
    GroupMessageEvent,
    MessageSegment,
    MessageEvent,
    Message,
    Bot,
)
from nonebot.exception import FinishedException, ActionFailed
from nonebot.params import EventMessage
from nonebot.permission import SUPERUSER
from .data_source import extract_member_at
from .database import MainTable
from .rule import message_at_rule
from .utils import node_custom, get_member_name

from nonebot.plugin import PluginMetadata

__plugin_meta__ = PluginMetadata(
    name="who_at_me",
    description="看看是谁又艾特了我",
    usage="直接发送 谁@我了？",
    extra={
        "author": "SEAFHMC <soku_ritsuki@outlook.com>",
        "version": "0.2.2",
    },
)

monitor = on_message(block=False, rule=message_at_rule)


async def create_record(bot: Bot, event: GroupMessageEvent, target_id):
    message = Message()
    if event.reply:
        message.append(MessageSegment.reply(event.reply.message_id))
    for segment in event.message:
        if segment.type == "at":
            card = get_member_name(
                await bot.get_group_member_info(
                    group_id=event.group_id, user_id=segment.data["qq"]
                )
            )
            message.append(f"@{card}")
            continue
        message.append(segment)

    MainTable.create(
        operator_id=event.user_id,
        operator_name=event.sender.card or event.sender.nickname,
        target_id=target_id,
        group_id=event.group_id,
        time=str(int(time.time())),
        message=message,
        message_id=event.message_id,
    )


@monitor.handle()
async def _(bot: Bot, event: GroupMessageEvent, message=EventMessage()):
    if event.reply:
        target_id = event.reply.sender.user_id
        await create_record(bot=bot, event=event, target_id=target_id)
        raise FinishedException
    if member_at := extract_member_at(message=message):
        for target_id in member_at:
            await create_record(bot=bot, event=event, target_id=target_id)
        raise FinishedException


who_at_me = on_regex(r"谁.*(@|艾特|圈|[aA][tT])+.?我")


@who_at_me.handle()
async def _(bot: Bot, event: MessageEvent):
    res_list: List[MainTable] = MainTable.select().where(
        MainTable.target_id == event.user_id
    )
    message_list: List[MessageSegment] = list()
    is_group = False
    for res in res_list:
        if is_group := isinstance(event, GroupMessageEvent):
            if res.group_id != event.group_id:
                continue
        message_list.append(
            node_custom(
                content=res.message,
                user_id=res.operator_id,
                name=res.operator_name,
                time=res.time,
            )
        )
        message_list.append(res.message)
    if not message_list:
        await who_at_me.finish(MessageSegment.reply(event.message_id) + "目前还没有人@您噢！")
    if is_group:
        event: GroupMessageEvent
        await bot.call_api(
            "send_group_forward_msg", group_id=event.group_id, messages=message_list
        )
    else:
        try:
            await bot.call_api(
                "send_private_forward_msg", user_id=event.user_id, messages=message_list
            )
        except ActionFailed as e:
            if "wording=API不存在" in (error := str(e)):
                nonebot.logger.error(
                    f"发送合并转发失败，请确认您的协议端支持私聊合并转发！(如果使用go-cqhttp，请确保版本号不小于v1.0.0-rc2)\n{error}"
                )
            else:
                raise e


clear_db = on_command("清除数据库", aliases={"clear_db", "db_clear", "已阅"})


@clear_db.handle()
async def _(event: MessageEvent):
    if isinstance(event, GroupMessageEvent):
        MainTable.delete().where(
            MainTable.target_id == event.user_id
            and MainTable.group_id == event.group_id
        ).execute()
        await clear_db.finish("已经清除您在本群的被艾特记录！")
    else:
        MainTable.delete().where(MainTable.target_id == event.user_id).execute()
        await clear_db.finish("已经清除您所有的被艾特记录！")


clear_db_all = on_command(
    "清除全部数据库", aliases={"clear_all", "db_clear --purge"}, permission=SUPERUSER
)


@clear_db_all.handle()
async def _():
    MainTable.delete().where(MainTable.target_id).execute()
    await clear_db.finish("已清理全部数据库")



from typing import Set

from nonebot.adapters.onebot.v11 import Message


def extract_member_at(message: Message) -> Set[str]:
    """提取消息中被艾特人的QQ号
    参数:
        message: 消息对象
    返回:
        被艾特列表
    """
    return {
        segment.data["qq"]
        for segment in message
        if (segment.type == "at") and ("qq" in segment.data)
    }



from nonebot.adapters.onebot.v11 import GroupMessageEvent, Message
from nonebot.params import EventMessage
from .data_source import extract_member_at


async def message_at_rule(event: GroupMessageEvent, message: Message = EventMessage()):
    return extract_member_at(message=message) or event.reply



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_who_at_me']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.3.1',
 'nonebot-adapter-onebot>=2.0.0-beta.1,<=2.1.0',
 'nonebot2>=2.0.0-beta.1,<=2.0.0-beta.4',
 'peewee>=3.14.4']

setup_kwargs = {
    'name': 'nonebot-plugin-who-at-me',
    'version': '0.2.2',
    'description': 'Find who on earth has ated you',
    'long_description': '<div align="center">\n  <a href="https://v2.nonebot.dev/store"><img src="https://s2.loli.net/2022/06/16/opBDE8Swad5rU3n.png" width="180" height="180" alt="NoneBotPluginLogo"></a>\n  <br>\n  <p><img src="https://s2.loli.net/2022/06/16/xsVUGRrkbn1ljTD.png" width="240" alt="NoneBotPluginText"></p>\n</div>\n\n<div align="center">\n\n# nonebot-plugin-who-at-me\n\n_✨ 看看是谁又在艾特我 ✨_\n</div>\n  \n # 说明\n 你是否遇到过这种情景：你点进一个99+的QQ群，发现有人艾特/回复过你，你满心期待地去查看，结果腾讯告诉你消息过多无法定义到上下文。现在你只需要部署一个机器人卧底即可找出到底是谁艾特了你。\n # 安装\n通过`pip`或`nb`安装；\n需要协议端支持转发合并消息。\n\n命令：\n```shell\npip install nonebot-plugin-who-at-me\n```\n```shell\nnb plugin install nonebot-plugin-who-at-me\n```\n# 配置\n记得配置SUPERUSER\n# 使用\n<div align="center">\n\n（这里默认COMMAND_START为"/"）\n| 命令              | 描述              |\n| ------------------ | --------------- |\n|/谁艾特我 | 查看到底是谁艾特了你       |\n|/clear_db     | 清理当前用户的消息记录 |\n|/clear_all     | 清理全部消息记录     |\n\n<del>你将获得一张劣质的表格】</del></br>\n没有图片！只有合并转发辣！\n',
    'author': 'SEAFHMC',
    'author_email': 'soku_ritsuki@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/SEAFHMC/nonebot-plugin-who-at-me',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Dict, Any

from nonebot.adapters.onebot.v11 import MessageSegment


def node_custom(
    user_id: int, name: str, time: int, content: MessageSegment
) -> "MessageSegment":
    return MessageSegment(
        type="node",
        data={"uin": str(user_id), "name": name, "content": content, "time": time},
    )


def get_member_name(member: Dict[str, Any]) -> "str":
    return member["card"] if not len(member["card"]) == 0 else member["nickname"]
